#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>
#include <sys/stat.h>

void set_uid (const char * fichier)
{
    struct stat etat;
    char *executable;
    char *blanc;

    /* Copie du nom du fichier pour pouvoir le modifier */
    if ((executable = (char *) malloc (strlen (fichier) + 1)) == NULL) {
        syslog (LOG_ERR, "Mmoire manquante");
        fprintf (stderr, "Mmoire manquante\n");
        exit (1);
    }
    strcpy (executable, fichier);
    /* Remplacement du premier blanc ou tabulation par un \0 */
    /* Ceci permet de supprimer les ventuels arguments */
    blanc = strchr (executable, ' ');
    if (blanc != NULL)
        blanc [0] = '\0';
    blanc = strchr (executable, '\t');
    if (blanc != NULL)
        blanc [0] = '\0';
 /* Lecture des informations concernant le fichier */
    if (stat (executable, & etat) != 0) {
        syslog (LOG_ERR, "Fichier %s inaccessible", executable);
        fprintf (stderr, "Fichier %s inaccessible\n", executable);
        free (executable);
        exit (1);
    }
    free (executable);

    /* On fixe les UID/GID effectifs et rels */
    if (setregid (etat . st_gid, etat . st_gid) < 0) perror ("setregid");    
    if (setreuid (etat . st_uid, etat . st_uid) < 0) perror ("setreuid");

    putenv ("HOME=/tmp");
}

void get_tty (const char * tty)
{
    int    fd;

    /* Ouverture du fichier indiqu */
    if ((fd = open (tty, O_RDWR, 0)) <0) {
        syslog (LOG_ERR, "Impossible d'ouvrir %s", tty);
        fprintf (stderr, "Impossible d'ouvrir %s uid %d \n", tty, geteuid ());
        exit (1);
    }
    /* Vrification qu'il s'agit bien d'un terminal */
    if (! isatty (fd)) {
    syslog (LOG_ERR, "%s n'est pas un terminal", tty);
    fprintf (stderr, "%s n'est pas un terminal\n", tty);
        exit (1);
    }
    /* Duplication du descripteur sur stdin, stdout, et stderr */
    if ((dup2 (fd, 0) < 0) || (dup2 (fd, 1) < 0) ||
    (dup2 (fd, 2) < 0)) {
        syslog (LOG_ERR, "Erreur dans dup2()");
        fprintf (stderr, "Erreur dans dup2()\n");
        exit (1);
    }
    close (fd);
}

int main (int argc, char * argv [])
{
    int retour;

    openlog (argv [0], 0, LOG_USER);

    if (argc != 3) {
     syslog (LOG_ERR, "Syntaxe : %s tty fichier", argv [0]);
     fprintf (stderr, "Syntaxe : %s tty fichier\n", argv [0]);
        exit (1);
    }
    /* Ouverture de la console indique en premier argument */
    get_tty (argv [1]);

    /* Utilisation des UID et GID du fichier excutable */
    set_uid (argv [2]);

    /* Lancement du programme */
    retour = system (argv [2]);

    switch (retour) {
        case 127 : fprintf (stderr, "/bin/sh -c %s failed\n", argv [2]);
                   break;
        case -1 :  fprintf (stderr, "Autre erreur\n");
                   break;
        default :  if (retour != 0) {
                       fprintf (stderr, "Code de retour de %s : %d\n", argv [2], retour);
                   }
                   break;

    }
    return (0);
}
